# Copyright 2017 - ZTE Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class Webhook(object):

    url = 'v1/webhook/'

    def __init__(self, api):
        self.api = api

    def list(self, all_tenants=False):
        """Get webhook list"""
        params = dict(all_tenants=all_tenants)
        return self.api.get(self.url, params=params).json()

    def show(self, id):
        """Show specific webhook

        :param id: id of registration to show
        """

        url = self.url + id
        return self.api.get(url).json()

    def add(self, url, regex_filter=None, headers=None):
        """Add a webhook to the DB


        :param url: url to register in the DB
        :param regex_filter: a optional regular expression dict to filter
        alarms
        :param headers: optional headers to attach to requests
        """

        params = dict(url=url, regex_filter=regex_filter,
                      headers=headers)

        return self.api.post(self.url, json=params).json()

    def delete(self, id):
        """delete a webhook from the DB


        :param id: id of webhook to delete
        """

        url = self.url + id

        return self.api.delete(url).json()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from cliff import lister
from cliff import show

from vitrageclient.common import utils


# noinspection PyAbstractClass
class ResourceShow(show.ShowOne):
    """Show a resource"""

    def get_parser(self, prog_name):
        parser = super(ResourceShow, self).get_parser(prog_name)
        parser.add_argument('vitrage_id', help='vitrage_id of a resource')
        return parser

    def take_action(self, parsed_args):
        vitrage_id = parsed_args.vitrage_id
        resource = utils.get_client(self).resource.get(vitrage_id=vitrage_id)
        return self.dict2columns(resource)


class ResourceList(lister.Lister):
    """List resources"""

    METADATA = ('name', 'project_id', 'update_timestamp')

    def get_parser(self, prog_name):
        parser = super(ResourceList, self).get_parser(prog_name)
        parser.add_argument('--type',
                            dest='resource_type',
                            metavar='<resource type>',
                            help='Type of resource')
        parser.add_argument('--all-tenants',
                            default=False,
                            dest='all_tenants',
                            action='store_true',
                            help='Shows resources of all the tenants')
        parser.add_argument('--filter',
                            metavar='<query>',
                            help='resource query')

        return parser

    def take_action(self, parsed_args):
        resource_type = parsed_args.resource_type
        all_tenants = parsed_args.all_tenants
        query = parsed_args.filter
        resources = utils.get_client(self).resource.list(
            resource_type=resource_type,
            all_tenants=all_tenants,
            query=query)
        # cluster, zone and host don't have "project_id" property
        # neutron.port don't have "name" property
        # cluster don't have "update_timestamp"
        for resource in resources:
            resource['metadata'] = \
                {item: resource[item] for item in self.METADATA
                 if item in resource}

        return utils.list2cols_with_rename(
            (
                ('ID', 'vitrage_id'),
                ('Type', 'vitrage_type'),
                ('Data Source ID', 'id'),
                ('State', 'vitrage_operational_state'),
                ('Metadata', 'metadata'),
            ), resources)


class ResourceCount(show.ShowOne):
    """Show a count of all resources"""

    def get_parser(self, prog_name):
        parser = super(ResourceCount, self).get_parser(prog_name)
        parser.add_argument('--type',
                            dest='resource_type',
                            metavar='<resource type>',
                            help='Type of resource')
        parser.add_argument('--all-tenants',
                            default=False,
                            dest='all_tenants',
                            action='store_true',
                            help='Shows resources of all the tenants')
        parser.add_argument('--filter',
                            metavar='<query>',
                            help='resource query'),
        parser.add_argument(
            '--group-by',
            dest='group_by',
            metavar='<group_by>',
            default='vitrage_type',
            help='A resource data field, to group by it\'s values'),
        return parser

    @property
    def formatter_default(self):
        return 'json'

    def take_action(self, parsed_args):
        resource_type = parsed_args.resource_type
        all_tenants = parsed_args.all_tenants
        query = parsed_args.filter
        group_by = parsed_args.group_by
        resource_count = utils.get_client(self).resource.count(
            resource_type=resource_type,
            all_tenants=all_tenants,
            query=query,
            group_by=group_by)

        return self.dict2columns(resource_count)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse

from cliff import show

from vitrageclient.common import utils
from vitrageclient import exceptions as exc


# noinspection PyAbstractClass
class TopologyShow(show.ShowOne):
    """Show the topology of the system"""

    @staticmethod
    def positive_non_zero_int(argument_value):
        if argument_value is None:
            return None
        try:
            value = int(argument_value)
        except ValueError:
            msg = "%s must be an integer" % argument_value
            raise argparse.ArgumentTypeError(msg)
        if value <= 0:
            msg = "%s must be greater than 0" % argument_value
            raise argparse.ArgumentTypeError(msg)
        return value

    def get_parser(self, prog_name):
        parser = super(TopologyShow, self).get_parser(prog_name)
        parser.add_argument('--filter',
                            metavar='<query>',
                            help='query for the graph)')

        parser.add_argument('--limit',
                            type=self.positive_non_zero_int,
                            metavar='<depth>',
                            help='the depth of the topology graph')

        parser.add_argument('--root',
                            help='the root of the topology graph')

        parser.add_argument('--graph-type',
                            choices=['tree', 'graph'],
                            default='graph',
                            dest='type',
                            help='graph type. '
                                 'Valid graph types: [tree, graph]')

        parser.add_argument('--all-tenants',
                            default=False,
                            dest='all_tenants',
                            action='store_true',
                            help='Shows entities of all the tenants in the '
                                 'entity graph')

        return parser

    @property
    def formatter_namespace(self):
        return 'vitrageclient.formatter.show'

    @property
    def formatter_default(self):
        return 'json'

    def take_action(self, parsed_args):
        limit = parsed_args.limit
        graph_type = parsed_args.type
        query = parsed_args.filter
        root = parsed_args.root
        all_tenants = parsed_args.all_tenants

        if graph_type == 'graph' and limit is not None and root is None:
            raise exc.CommandError("Graph-type 'graph' "
                                   "requires a 'root' with 'limit'.")

        topology = utils.get_client(self).topology.get(limit=limit,
                                                       graph_type=graph_type,
                                                       query=query,
                                                       root=root,
                                                       all_tenants=all_tenants)
        return self.dict2columns(topology)



# Copyright 2016 - Nokia Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

from cliff import command
from cliff import lister
from cliff import show
from oslo_log import log

from vitrageclient.common import utils
from vitrageclient.common.utils import find_template_with_uuid

LOG = log.getLogger(__name__)


def _parse_template_params(cli_param_list):
    return dict(cli_param.split('=', 1) for cli_param in cli_param_list) \
        if cli_param_list else {}


# noinspection PyAbstractClass
class TemplateValidate(show.ShowOne):
    """Validate a template file"""

    def get_parser(self, prog_name):
        parser = super(TemplateValidate, self).get_parser(prog_name)
        parser.add_argument('--path',
                            required=True,
                            help='full path for template file or templates dir'
                            )
        parser.add_argument('--type',
                            choices=['standard', 'definition', 'equivalence'],
                            help='Template type. Valid types:'
                                 '[standard, definition, equivalence]')
        parser.add_argument('--params', nargs='+',
                            help='Actual values for parameters of the '
                                 'template. Several key=value pairs may be '
                                 'used, for example: --params '
                                 'template_name=cpu_problem '
                                 'alarm_name=\'High CPU Load\'')
        return parser

    @property
    def formatter_default(self):
        return 'json'

    def take_action(self, parsed_args):
        cli_param_list = parsed_args.params
        params = _parse_template_params(cli_param_list)

        result = utils.get_client(self).template.validate(
            path=parsed_args.path,
            template_type=parsed_args.type,
            params=params)

        return self.dict2columns(result)


class TemplateVersions(lister.Lister):
    """List all template versions"""

    def get_parser(self, prog_name):
        parser = super(TemplateVersions, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        templates = utils.get_client(self).template.versions()
        return utils.list2cols_with_rename(
            (
                ('Version', 'version'),
                ('Status', 'status'),
            ), templates)


class TemplateList(lister.Lister):
    """List all templates"""

    def get_parser(self, prog_name):
        parser = super(TemplateList, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        templates = utils.get_client(self).template.list()
        return utils.list2cols_with_rename(
            (
                ('UUID', 'uuid'),
                ('Name', 'name'),
                ('Status', 'status'),
                ('Status details', 'status details'),
                ('Date', 'date'),
                ('Type', 'type'),
            ), templates)


class TemplateShow(show.ShowOne):
    """Show a template"""

    def get_parser(self, prog_name):
        parser = super(TemplateShow, self).get_parser(prog_name)
        parser.add_argument('id', help='Template UUID or Name')
        return parser

    @property
    def formatter_default(self):
        return 'yaml'

    def take_action(self, parsed_args):
        _id = parsed_args.id
        template = utils.get_client(self).template.show(_id=_id)
        return self.dict2columns(template)


class TemplateAdd(lister.Lister):
    """Add a template

    support 3 types of templates:
    standard, definition, equivalence
    """

    def get_parser(self, prog_name):
        parser = super(TemplateAdd, self).get_parser(prog_name)
        parser.add_argument('--path',
                            required=True,
                            help='full path for template file or templates dir'
                            )
        parser.add_argument('--type',
                            choices=['standard', 'definition', 'equivalence'],
                            help='Template type. Valid types:'
                                 '[standard, definition, equivalence]')
        parser.add_argument('--params', nargs='+',
                            help='Actual values for parameters of the '
                                 'template. Several key=value pairs may be '
                                 'used, for example: --params '
                                 'template_name=cpu_problem '
                                 'alarm_name=\'High CPU Load\'')

        parser.add_argument('--wait',
                            type=int,
                            default=None,
                            nargs='?',
                            const=sys.maxsize,
                            help='Wait until template is ACTIVE or in ERROR'
                                 ' default is to wait forever '
                                 'else number of seconds'
                            )

        parser.add_argument('--overwrite',
                            default=False,
                            action='store_true',
                            help='If template exists by name overwrite it'
                            )

        return parser

    def take_action(self, parsed_args):
        path = parsed_args.path
        template_type = parsed_args.type
        cli_param_list = parsed_args.params
        params = _parse_template_params(cli_param_list)
        wait = parsed_args.wait
        overwrite = parsed_args.overwrite

        templates = utils.get_client(self).template.add(
            path=path, template_type=template_type, params=params,
            overwrite=overwrite)

        if wait:
            utils.wait_for_action_to_end(wait,
                                         self._check_finished_loading,
                                         templates=templates)

        return utils.list2cols_with_rename(
            (
                ('UUID', 'uuid'),
                ('Name', 'name'),
                ('Status', 'status'),
                ('Status details', 'status details'),
                ('Date', 'date'),
                ('Type', 'type'),
            ), templates)

    def _check_finished_loading(self, templates):
        if all(
                (
                    template['status'] == 'ERROR'
                    for template in templates
                )
        ):
            return True

        try:
            api_templates = utils.get_client(self).template.list()
            self._update_templates_status(api_templates, templates)
            if any(
                    (
                        template['status'] == 'LOADING'
                        for template in templates
                    )
            ):
                return False
            return True
        except Exception:
            return True

    @staticmethod
    def _update_templates_status(api_templates, templates):
        for template in templates:
            uuid = template.get('uuid')
            if uuid:
                api_template = find_template_with_uuid(uuid, api_templates)
                if api_template:
                    template['status'] = api_template['status']


class TemplateDelete(command.Command):
    """Delete a template"""

    def get_parser(self, prog_name):
        parser = super(TemplateDelete, self).get_parser(prog_name)
        parser.add_argument('id',
                            help='<ID or Name> of a template',
                            nargs='+')
        parser.add_argument('--wait',
                            type=int,
                            default=None,
                            nargs='?',
                            const=sys.maxsize,
                            help='Wait until template is DELETED or in ERROR'
                            ' default is to wait forever else number of '
                                 'seconds'
                            )
        return parser

    @property
    def formatter_default(self):
        return 'json'

    def take_action(self, parsed_args):
        ids = parsed_args.id
        wait = parsed_args.wait
        utils.get_client(self).template.delete(ids=ids)
        if wait:
            utils.wait_for_action_to_end(wait,
                                         self._check_deleted,
                                         ids=ids)

    def _check_deleted(self, ids):
        for _id in ids:
            try:
                utils.get_client(self).template.show(_id)
            except Exception:  # if deleted we get exception
                pass
            else:
                return False
        return True



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class Alarm(object):
    url = 'v1/alarm/'

    def __init__(self, api):
        self.api = api

    def list(self, vitrage_id, all_tenants=False,
             limit=1000,
             sort_by=['start_timestamp', 'vitrage_id'],
             sort_dirs=['asc', 'asc'],
             filter_by=None,
             filter_vals=None,
             next_page=True,
             marker=None):
        """Get a all alarms on entity

        :param all_tenants: should return all tenants alarms
        :param vitrage_id: the id for the entity
        :param limit: number of rows in page.
        :param sort_by: array of attributes by which results should be sorted
        :param sort_dirs: per-column array of sort_dirs,
        corresponding to sort_keys ('asc' or 'desc')
        :param filter_by: array of attributes by which results will be filtered
        :param filter_vals: array of filter values corresponding to filter_by
        :param next_page: True if next page was requested, False if previouse
        :param marker: if next_page is True,
        vitrage_id of last row in current page is needed,
        else, vitrage_id of first row in current page.
        marker=None if the request is for the first page.
        """

        params = dict(vitrage_id=vitrage_id,
                      all_tenants=all_tenants,
                      limit=limit,
                      sort_by=sort_by,
                      sort_dirs=sort_dirs,
                      filter_by=filter_by,
                      filter_vals=filter_vals,
                      next_page=next_page,
                      marker=marker
                      )
        return self.api.get(self.url, params=params).json()

    def history(self, all_tenants=False,
                start=None,
                end=None,
                limit=1000,
                sort_by=('start_timestamp', 'vitrage_id'),
                sort_dirs=('asc', 'asc'),
                filter_by=None,
                filter_vals=None,
                next_page=True,
                marker=None):
        """Get the alarm history

        :param all_tenants: should return all tenants alarms
        :param start: start of time frame.
        :param end: end of time frame.
        :param limit: number of rows in page.
        :param sort_by: array of attributes by which results should be sorted
        :param sort_dirs: per-column array of sort_dirs,
        corresponding to sort_keys ('asc' or 'desc')
        :param filter_by: array of attributes by which results will be filtered
        :param filter_vals: array of filter values corresponding to filter_by
        :param next_page: True if next page was requested, False if previouse
        :param marker: if next_page is True,
        vitrage_id of last row in current page is needed,
        else, vitrage_id of first row in current page.
        marker=None if the request is for the first page.
        """

        params = dict(all_tenants=all_tenants,
                      start=start,
                      end=end,
                      limit=limit,
                      sort_by=sort_by,
                      sort_dirs=sort_dirs,
                      filter_by=filter_by,
                      filter_vals=filter_vals,
                      next_page=next_page,
                      marker=marker
                      )
        return self.api.get(self.url + 'history/', params=params).json()

    def get(self, vitrage_id):
        """Get an alarm

        :param vitrage_id: the vitrage_id of the alarm
        """
        url = self.url + vitrage_id
        return self.api.get(url).json()

    def count(self, all_tenants=False):
        """Get a count of all alarms present

        :param all_tenants: should return all tenants alarms
        """
        params = dict(all_tenants=all_tenants)
        return self.api.get(self.url + 'count/', params=params).json()



# Copyright 2017 Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from argparse import ArgumentParser
from argparse import ArgumentTypeError
from unittest import mock

import json

# noinspection PyPackageRequirements
# noinspection PyPackageRequirements
import six
from testtools import ExpectedException

from vitrageclient.common.formatters import DOTFormatter
from vitrageclient.common.formatters import GraphMLFormatter
from vitrageclient.tests.cli.base import CliTestCase
from vitrageclient.v1.cli.topology import TopologyShow

JSON_DATA = '''
{
  "directed": true,
  "graph": {},
  "links": [
    {
      "key": "contains",
      "source": 0,
      "vitrage_is_deleted": false,
      "relationship_type": "contains",
      "target": 2
    },
    {
      "key": "contains",
      "source": 1,
      "vitrage_is_deleted": false,
      "relationship_type": "contains",
      "target": 0
    }
  ],
  "multigraph": true,
  "nodes": [
    {
      "id": "nova",
      "vitrage_sample_timestamp": "2018-12-31T13:44:03Z",
      "vitrage_datasource_name": "nova.zone",
      "vitrage_operational_state": "OK",
      "name": "nova",
      "state": "available",
      "update_timestamp": "2018-12-31T13:44:03Z",
      "is_real_vitrage_id": true,
      "vitrage_id": "05a19de3-e929-4730-ad81-10fa57dcfa0a",
      "vitrage_aggregated_state": "AVAILABLE",
      "vitrage_type": "nova.zone",
      "vitrage_is_deleted": false,
      "graph_index": 0,
      "vitrage_category": "RESOURCE",
      "vitrage_cached_id": "125f1d8c4451a6385cc2cfa2b0ba45be",
      "vitrage_is_placeholder": false
    },
    {
      "id": "OpenStack Cluster",
      "vitrage_sample_timestamp": "2018-12-31T13:44:03Z",
      "vitrage_operational_state": "OK",
      "name": "openstack.cluster",
      "state": "available",
      "graph_index": 1,
      "vitrage_id": "070c413e-5a8c-4823-ae20-af44936de2a0",
      "vitrage_aggregated_state": "AVAILABLE",
      "vitrage_type": "openstack.cluster",
      "is_real_vitrage_id": true,
      "vitrage_category": "RESOURCE",
      "vitrage_cached_id": "3c7f9d22d9dd1615a00404f86cb3e289",
      "vitrage_is_deleted": false,
      "vitrage_is_placeholder": false
    },
    {
      "id": "ebarilan-devstack",
      "vitrage_sample_timestamp": "2018-12-31T13:44:03Z",
      "vitrage_datasource_name": "nova.host",
      "vitrage_operational_state": "OK",
      "name": "ebarilan-devstack",
      "state": "available",
      "update_timestamp": "2018-12-31T13:44:03Z",
      "is_real_vitrage_id": true,
      "vitrage_id": "10da4fa2-397f-4b2e-a43b-937e11ab7daf",
      "vitrage_aggregated_state": "AVAILABLE",
      "vitrage_type": "nova.host",
      "vitrage_is_deleted": false,
      "graph_index": 2,
      "vitrage_category": "RESOURCE",
      "vitrage_cached_id": "9ae4db6fb920e19cb5c57a428b29eb59",
      "vitrage_is_placeholder": false
    },
    {
      "id": "b36b4d7a-b309-4b02-9662-5abd79741750",
      "vitrage_sample_timestamp": "2018-12-31T13:44:04Z",
      "vitrage_datasource_name": "cinder.volume",
      "project_id": "210140f1f5a94af99e0adf79a883b75a",
      "vitrage_operational_state": "OK",
      "vitrage_aggregated_state": "AVAILABLE",
      "vitrage_is_placeholder": false,
      "state": "available",
      "attachments": [],
      "graph_index": 3,
      "vitrage_id": "f0ca9fac-3ebd-4748-97ba-e93a7e7108aa",
      "size": 1,
      "vitrage_type": "cinder.volume",
      "vitrage_is_deleted": false,
      "vitrage_category": "RESOURCE",
      "vitrage_cached_id": "f998c5f7bf1851e17e3eea902800a7df",
      "update_timestamp": "2018-12-31T08:43:32Z",
      "is_real_vitrage_id": true,
      "volume_type": "lvmdriver-1"
    },
    {
      "id": "cebf5d5b-d7b1-4cfb-86fa-f660306b4c1a",
      "vitrage_sample_timestamp": "2018-12-31T13:44:04Z",
      "vitrage_datasource_name": "neutron.network",
      "project_id": "210140f1f5a94af99e0adf79a883b75a",
      "vitrage_operational_state": "OK",
      "vitrage_category": "RESOURCE",
      "vitrage_is_placeholder": false,
      "state": "ACTIVE",
      "update_timestamp": "2018-12-30T08:30:33Z",
      "is_real_vitrage_id": true,
      "vitrage_id": "eea46e33-81dc-4430-a771-852bac37b43d",
      "vitrage_aggregated_state": "ACTIVE",
      "vitrage_type": "neutron.network",
      "vitrage_is_deleted": false,
      "graph_index": 4,
      "name": "public",
      "vitrage_cached_id": "a0eeca0ab2c865915e23319a2e6d0fd7"
    }
  ],
  "raw": true
}

'''
DOT_DATA = '''\
strict digraph  {
0 [id=nova, is_real_vitrage_id=True, label="nova\\nnova.zone", state=available, update_timestamp="2018-12-31T13:44:03Z", vitrage_aggregated_state=AVAILABLE, vitrage_cached_id="125f1d8c4451a6385cc2cfa2b0ba45be", vitrage_category=RESOURCE, vitrage_datasource_name="nova.zone", vitrage_id="05a19de3-e929-4730-ad81-10fa57dcfa0a", vitrage_is_deleted=False, vitrage_is_placeholder=False, vitrage_operational_state=OK, vitrage_sample_timestamp="2018-12-31T13:44:03Z", vitrage_type="nova.zone"];
1 [id="OpenStack Cluster", is_real_vitrage_id=True, label="openstack.cluster", state=available, vitrage_aggregated_state=AVAILABLE, vitrage_cached_id="3c7f9d22d9dd1615a00404f86cb3e289", vitrage_category=RESOURCE, vitrage_id="070c413e-5a8c-4823-ae20-af44936de2a0", vitrage_is_deleted=False, vitrage_is_placeholder=False, vitrage_operational_state=OK, vitrage_sample_timestamp="2018-12-31T13:44:03Z", vitrage_type="openstack.cluster"];
2 [id="ebarilan-devstack", is_real_vitrage_id=True, label="ebarilan-devstack\\nnova.host", state=available, update_timestamp="2018-12-31T13:44:03Z", vitrage_aggregated_state=AVAILABLE, vitrage_cached_id="9ae4db6fb920e19cb5c57a428b29eb59", vitrage_category=RESOURCE, vitrage_datasource_name="nova.host", vitrage_id="10da4fa2-397f-4b2e-a43b-937e11ab7daf", vitrage_is_deleted=False, vitrage_is_placeholder=False, vitrage_operational_state=OK, vitrage_sample_timestamp="2018-12-31T13:44:03Z", vitrage_type="nova.host"];
3 [attachments="[]", id="b36b4d7a-b309-4b02-9662-5abd79741750", is_real_vitrage_id=True, label="cinder.volume", project_id="210140f1f5a94af99e0adf79a883b75a", size=1, state=available, update_timestamp="2018-12-31T08:43:32Z", vitrage_aggregated_state=AVAILABLE, vitrage_cached_id=f998c5f7bf1851e17e3eea902800a7df, vitrage_category=RESOURCE, vitrage_datasource_name="cinder.volume", vitrage_id="f0ca9fac-3ebd-4748-97ba-e93a7e7108aa", vitrage_is_deleted=False, vitrage_is_placeholder=False, vitrage_operational_state=OK, vitrage_sample_timestamp="2018-12-31T13:44:04Z", vitrage_type="cinder.volume", volume_type="lvmdriver-1"];
4 [id="cebf5d5b-d7b1-4cfb-86fa-f660306b4c1a", is_real_vitrage_id=True, label="public\\nneutron.network", project_id="210140f1f5a94af99e0adf79a883b75a", state=ACTIVE, update_timestamp="2018-12-30T08:30:33Z", vitrage_aggregated_state=ACTIVE, vitrage_cached_id=a0eeca0ab2c865915e23319a2e6d0fd7, vitrage_category=RESOURCE, vitrage_datasource_name="neutron.network", vitrage_id="eea46e33-81dc-4430-a771-852bac37b43d", vitrage_is_deleted=False, vitrage_is_placeholder=False, vitrage_operational_state=OK, vitrage_sample_timestamp="2018-12-31T13:44:04Z", vitrage_type="neutron.network"];
0 -> 2  [label=contains, vitrage_is_deleted=False];
1 -> 0  [label=contains, vitrage_is_deleted=False];
}
'''  # noqa

GRAPHML_DATA = u'''\
<?xml version='1.0' encoding='utf-8'?>
<graphml xmlns="http://graphml.graphdrawing.org/xmlns" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://graphml.graphdrawing.org/xmlns http://graphml.graphdrawing.org/xmlns/1.0/graphml.xsd">
  <key attr.name="size" attr.type="int" for="node" id="d20" />
  <key attr.name="project_id" attr.type="string" for="node" id="d19" />
  <key attr.name="volume_type" attr.type="string" for="node" id="d18" />
  <key attr.name="attachments" attr.type="string" for="node" id="d17" />
  <key attr.name="label" attr.type="string" for="edge" id="d16" />
  <key attr.name="vitrage_is_deleted" attr.type="boolean" for="edge" id="d15" />
  <key attr.name="is_real_vitrage_id" attr.type="boolean" for="node" id="d14" />
  <key attr.name="id" attr.type="string" for="node" id="d13" />
  <key attr.name="label" attr.type="string" for="node" id="d12" />
  <key attr.name="vitrage_is_placeholder" attr.type="boolean" for="node" id="d11" />
  <key attr.name="vitrage_aggregated_state" attr.type="string" for="node" id="d10" />
  <key attr.name="vitrage_sample_timestamp" attr.type="string" for="node" id="d9" />
  <key attr.name="vitrage_type" attr.type="string" for="node" id="d8" />
  <key attr.name="vitrage_cached_id" attr.type="string" for="node" id="d7" />
  <key attr.name="state" attr.type="string" for="node" id="d6" />
  <key attr.name="vitrage_operational_state" attr.type="string" for="node" id="d5" />
  <key attr.name="vitrage_datasource_name" attr.type="string" for="node" id="d4" />
  <key attr.name="vitrage_category" attr.type="string" for="node" id="d3" />
  <key attr.name="update_timestamp" attr.type="string" for="node" id="d2" />
  <key attr.name="vitrage_is_deleted" attr.type="boolean" for="node" id="d1" />
  <key attr.name="vitrage_id" attr.type="string" for="node" id="d0" />
  <graph edgedefault="directed">
    <node id="0">
      <data key="d0">05a19de3-e929-4730-ad81-10fa57dcfa0a</data>
      <data key="d1">False</data>
      <data key="d2">2018-12-31T13:44:03Z</data>
      <data key="d3">RESOURCE</data>
      <data key="d4">nova.zone</data>
      <data key="d5">OK</data>
      <data key="d6">available</data>
      <data key="d7">125f1d8c4451a6385cc2cfa2b0ba45be</data>
      <data key="d8">nova.zone</data>
      <data key="d9">2018-12-31T13:44:03Z</data>
      <data key="d10">AVAILABLE</data>
      <data key="d11">False</data>
      <data key="d12">nova
nova.zone</data>
      <data key="d13">nova</data>
      <data key="d14">True</data>
    </node>
    <node id="1">
      <data key="d0">070c413e-5a8c-4823-ae20-af44936de2a0</data>
      <data key="d1">False</data>
      <data key="d3">RESOURCE</data>
      <data key="d12">openstack.cluster</data>
      <data key="d5">OK</data>
      <data key="d6">available</data>
      <data key="d7">3c7f9d22d9dd1615a00404f86cb3e289</data>
      <data key="d8">openstack.cluster</data>
      <data key="d9">2018-12-31T13:44:03Z</data>
      <data key="d10">AVAILABLE</data>
      <data key="d11">False</data>
      <data key="d13">OpenStack Cluster</data>
      <data key="d14">True</data>
    </node>
    <node id="2">
      <data key="d0">10da4fa2-397f-4b2e-a43b-937e11ab7daf</data>
      <data key="d1">False</data>
      <data key="d2">2018-12-31T13:44:03Z</data>
      <data key="d3">RESOURCE</data>
      <data key="d4">nova.host</data>
      <data key="d5">OK</data>
      <data key="d6">available</data>
      <data key="d7">9ae4db6fb920e19cb5c57a428b29eb59</data>
      <data key="d8">nova.host</data>
      <data key="d9">2018-12-31T13:44:03Z</data>
      <data key="d10">AVAILABLE</data>
      <data key="d11">False</data>
      <data key="d12">ebarilan-devstack
nova.host</data>
      <data key="d13">ebarilan-devstack</data>
      <data key="d14">True</data>
    </node>
    <node id="3">
      <data key="d0">f0ca9fac-3ebd-4748-97ba-e93a7e7108aa</data>
      <data key="d1">False</data>
      <data key="d2">2018-12-31T08:43:32Z</data>
      <data key="d17">[]</data>
      <data key="d3">RESOURCE</data>
      <data key="d18">lvmdriver-1</data>
      <data key="d4">cinder.volume</data>
      <data key="d5">OK</data>
      <data key="d6">available</data>
      <data key="d7">f998c5f7bf1851e17e3eea902800a7df</data>
      <data key="d8">cinder.volume</data>
      <data key="d9">2018-12-31T13:44:04Z</data>
      <data key="d12">cinder.volume</data>
      <data key="d10">AVAILABLE</data>
      <data key="d11">False</data>
      <data key="d19">210140f1f5a94af99e0adf79a883b75a</data>
      <data key="d13">b36b4d7a-b309-4b02-9662-5abd79741750</data>
      <data key="d14">True</data>
      <data key="d20">1</data>
    </node>
    <node id="4">
      <data key="d0">eea46e33-81dc-4430-a771-852bac37b43d</data>
      <data key="d1">False</data>
      <data key="d2">2018-12-30T08:30:33Z</data>
      <data key="d3">RESOURCE</data>
      <data key="d4">neutron.network</data>
      <data key="d5">OK</data>
      <data key="d6">ACTIVE</data>
      <data key="d7">a0eeca0ab2c865915e23319a2e6d0fd7</data>
      <data key="d8">neutron.network</data>
      <data key="d9">2018-12-31T13:44:04Z</data>
      <data key="d12">public
neutron.network</data>
      <data key="d10">ACTIVE</data>
      <data key="d11">False</data>
      <data key="d19">210140f1f5a94af99e0adf79a883b75a</data>
      <data key="d13">cebf5d5b-d7b1-4cfb-86fa-f660306b4c1a</data>
      <data key="d14">True</data>
    </node>
    <edge source="0" target="2">
      <data key="d15">False</data>
      <data key="d16">contains</data>
    </edge>
    <edge source="1" target="0">
      <data key="d15">False</data>
      <data key="d16">contains</data>
    </edge>
  </graph>
</graphml>
'''  # noqa


# noinspection PyAttributeOutsideInit
class TopologyShowTest(CliTestCase):

    def setUp(self):
        super(TopologyShowTest, self).setUp()
        self.topology_show = TopologyShow(mock.Mock(), mock.Mock())

    def test_positive_integer_validation_with_negative(self):
        self.assertRaises(ArgumentTypeError,
                          self.topology_show.positive_non_zero_int, -1)

    def test_positive_integer_validation_with_zero(self):
        self.assertRaises(ArgumentTypeError,
                          self.topology_show.positive_non_zero_int, 0)

    def test_positive_integer_validation_with_string(self):
        self.assertRaises(ArgumentTypeError,
                          self.topology_show.positive_non_zero_int, 'bla')

    def test_positive_integer_validation_with_positive(self):
        self.topology_show.positive_non_zero_int(1)

    @mock.patch.object(ArgumentParser, "error")
    def test_parser_topology_limit_with_a_negative_number(self, mock_parser):
        mock_parser.side_effect = self._my_parser_error_func
        parser = self.topology_show.get_parser('vitrage topology show')

        with ExpectedException(ArgumentTypeError,
                               'argument --limit: -5 must be greater than 0'):
            parser.parse_args(args=['--filter', 'bla',
                                    '--limit', '-5',
                                    '--root', 'blabla',
                                    '--graph-type', 'tree'])

    @mock.patch.object(ArgumentParser, "error")
    def test_parser_topology_limit_with_a_string(self, mock_parser):
        mock_parser.side_effect = self._my_parser_error_func
        parser = self.topology_show.get_parser('vitrage topology show')

        with ExpectedException(ArgumentTypeError,
                               'argument --limit: spam must be an integer'):
            parser.parse_args(args=['--filter', 'bla',
                                    '--limit', 'spam',
                                    '--root', 'blabla',
                                    '--graph-type', 'tree'])

    def test_dot_emitter(self):
        def dict2columns(data):
            return zip(*sorted(data.items()))

        out = six.StringIO()
        formatter = DOTFormatter()
        topology = json.loads(JSON_DATA)
        columns, topology = dict2columns(topology)

        formatter.emit_one(columns, topology, out)

        self.assertEqual(DOT_DATA, out.getvalue())

    def test_graphml_emitter(self):
        def dict2columns(data):
            return zip(*sorted(data.items()))

        out = six.BytesIO()
        formatter = GraphMLFormatter()
        topology = json.loads(JSON_DATA)
        columns, topology = dict2columns(topology)

        formatter.emit_one(columns, topology, out)

        actual = out.getvalue().decode('utf-8')  # noqa

    # skipping this. graphml keeps changing the xml file
    # from test to test I cannot compare
    # for now I will just check that it can parse and output
    # and xml file
    #  self.assertEqual(GRAPHML_DATA, actual)



# Copyright 2016 Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""
Vitrage command line interface
"""

import logging
import os
import sys
import warnings

from cliff import app
from cliff import commandmanager
from keystoneauth1 import loading
from oslo_utils import importutils

from vitrageclient import __version__
from vitrageclient import auth
from vitrageclient import client

from vitrageclient.v1.cli import alarm
from vitrageclient.v1.cli import event
from vitrageclient.v1.cli import healthcheck
from vitrageclient.v1.cli import rca
from vitrageclient.v1.cli import resource
from vitrageclient.v1.cli import service
from vitrageclient.v1.cli import status
from vitrageclient.v1.cli import template
from vitrageclient.v1.cli import topology
from vitrageclient.v1.cli import webhook


profiler = importutils.try_import('osprofiler.profiler')


class VitrageCommandManager(commandmanager.CommandManager):
    COMMANDS = {
        'topology show': topology.TopologyShow,
        'resource show': resource.ResourceShow,
        'resource list': resource.ResourceList,
        'resource count': resource.ResourceCount,
        'alarm list': alarm.AlarmList,
        'alarm history': alarm.AlarmHistory,
        'alarm show': alarm.AlarmShow,
        'alarm count': alarm.AlarmCount,
        'rca show': rca.RcaShow,
        'template validate': template.TemplateValidate,
        'template list': template.TemplateList,
        'template versions': template.TemplateVersions,
        'template show': template.TemplateShow,
        'template add': template.TemplateAdd,
        'template delete': template.TemplateDelete,
        'event post': event.EventPost,
        'healthcheck': healthcheck.HealthCheck,
        'webhook delete': webhook.WebhookDelete,
        'webhook add': webhook.WebhookAdd,
        'webhook list': webhook.WebhookList,
        'webhook show': webhook.WebhookShow,
        'service list': service.ServiceList,
        'status': status.Status,
    }

    def load_commands(self, namespace):
        for k, v in self.COMMANDS.items():
            self.add_command(k, v)


class VitrageShell(app.App):
    def __init__(self):
        super(VitrageShell, self).__init__(
            description=__doc__,
            version=__version__,
            command_manager=VitrageCommandManager('vitrageclient'),
            deferred_help=True)

        self._client = None

    def build_option_parser(self, description, version, **argparse_kwargs):
        """Return an argparse option parser for this application.

        Subclasses may override this method to extend
        the parser with more global options.

        :param description: full description of the application
        :paramtype description: str
        :param version: version number for the application
        :paramtype version: str
        :param argparse_kwargs: extra keyword argument passed to the
                                ArgumentParser constructor
        :paramtype extra_kwargs: dict
        """

        parser = super(VitrageShell, self).build_option_parser(
            description, version, argparse_kwargs={'allow_abbrev': False})

        parser.add_argument('--vitrage-api-version',
                            default=os.environ.get('VITRAGE_API_VERSION', '1'),
                            help='Defaults to env[VITRAGE_API_VERSION] or 1.')

        if profiler:
            parser.add_argument(
                '--profile',
                metavar='HMAC_KEY',
                default=os.environ.get('OS_PROFILE'),
                help='HMAC key to use for encrypting context '
                     'data for performance profiling of request. '
                     'This key should be the value of the HMAC '
                     'key configured for the osprofiler '
                     'middleware in Vitrage api; it is specified '
                     'in the Vitrage configuration file at'
                     '"/etc/vitrage/vitrage.conf". Without the '
                     'key, profiling will not be triggered even '
                     'if osprofiler is enabled on the server '
                     'side.')

        plugin = self.register_keyauth_argparse_arguments(parser)

        if not isinstance(plugin, (auth.VitrageNoAuthLoader,
                                   auth.VitrageKeycloakLoader)):
            parser.add_argument('--endpoint',
                                default=os.environ.get('VITRAGE_ENDPOINT'),
                                help='Vitrage endpoint (Env: VITRAGE_ENDPOINT)'
                                )

        return parser

    @staticmethod
    def register_keyauth_argparse_arguments(parser):
        parser.add_argument(
            '--os-region-name',
            metavar='<auth-region-name>',
            dest='region_name',
            default=os.environ.get('OS_REGION_NAME'),
            help='Authentication region name (Env: OS_REGION_NAME)')
        parser.add_argument(
            '--os-interface',
            metavar='<interface>',
            dest='interface',
            choices=['admin', 'public', 'internal'],
            default=os.environ.get('OS_INTERFACE'),
            help='Select an interface type.'
                 ' Valid interface types: [admin, public, internal].'
                 ' (Env: OS_INTERFACE)')

        loading.register_session_argparse_arguments(parser=parser)

        return loading.register_auth_argparse_arguments(parser=parser,
                                                        argv=sys.argv,
                                                        default='password')

    @property
    def client(self):
        if self._client is None:
            if hasattr(self.options, "endpoint"):
                endpoint_override = self.options.endpoint
            else:
                endpoint_override = None

            if profiler and self.options.profile:
                # Initialize the root of the future trace: the created trace
                # ID will be used as the very first parent to which all
                # related traces will be bound to. The given HMAC key must
                # correspond to the one set in vitrage api vitrage.conf,
                # otherwise the latter will fail to check the request
                # signature and will skip initialization of osprofiler on the
                # server side.
                profiler.init(self.options.profile)

            auth_plugin = loading.load_auth_from_argparse_arguments(
                self.options)
            session = loading.load_session_from_argparse_arguments(
                self.options, auth=auth_plugin)

            # noinspection PyAttributeOutsideInit
            self._client = client.Client(
                self.options.vitrage_api_version,
                session=session,
                interface=self.options.interface,
                region_name=self.options.region_name,
                endpoint_override=endpoint_override)

        return self._client

    def configure_logging(self):
        if self.options.debug:
            self._set_debug_logging_messages()

        super(VitrageShell, self).configure_logging()
        root_logger = logging.getLogger('')

        self._configure_logging_messages(root_logger)
        self._hide_useless_logging_messages()

    def _set_debug_logging_messages(self):
        self.options.verbose_level = 3

    def _configure_logging_messages(self, root_logger):
        if self.options.verbose_level == 0:
            self._set_quiet_logging_messages(root_logger)
        elif self.options.verbose_level == 1:
            self._set_default_logging_messages(root_logger)
        elif self.options.verbose_level == 2:
            self._set_verbose_logging_messages(root_logger)
        elif self.options.verbose_level >= 3:
            self._set_double_verbose_logging_messages(root_logger)

    @staticmethod
    def _set_double_verbose_logging_messages(root_logger):
        root_logger.setLevel(logging.DEBUG)

    @staticmethod
    def _set_verbose_logging_messages(root_logger):
        root_logger.setLevel(logging.INFO)
        warnings.simplefilter("once")

    @staticmethod
    def _set_default_logging_messages(root_logger):
        root_logger.setLevel(logging.WARNING)
        warnings.simplefilter("ignore")

    @staticmethod
    def _set_quiet_logging_messages(root_logger):
        root_logger.setLevel(logging.ERROR)
        warnings.simplefilter("ignore")

    def _hide_useless_logging_messages(self):
        requests_log = logging.getLogger('requests')
        cliff_log = logging.getLogger('cliff')
        stevedore_log = logging.getLogger('stevedore')
        iso8601_log = logging.getLogger('iso8601')
        cliff_log.setLevel(logging.ERROR)
        stevedore_log.setLevel(logging.ERROR)
        iso8601_log.setLevel(logging.ERROR)
        if self.options.debug:
            requests_log.setLevel(logging.DEBUG)
        else:
            requests_log.setLevel(logging.ERROR)


def main(args=None):
    try:
        if args is None:
            args = sys.argv[1:]
        result = VitrageShell().run(args)
        if profiler and '--profile' in args:
            trace_id = profiler.get().get_base_id()
            print("\n\nTo display trace use the command:\n\n"
                  "  osprofiler trace show --html %s " % trace_id)
        return result
    except KeyboardInterrupt:
        print('... terminating vitrage client', file=sys.stderr)
        sys.exit(1)
    except Exception as e:
        print(e)
        sys.exit(1)


if __name__ == '__main__':
    main()



# Copyright 2016 - Nokia Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import time


def args_to_dict(args, attrs):
    return {(attr, value)
            for attr, value in [(attr, getattr(args, attr)) for attr in attrs]
            if value is not None}


def list2cols(cols, objs):
    return cols, [tuple([o[k] for k in cols])
                  for o in objs]


def list2cols_with_rename(names_and_keys, objs):
    cols = [i[0] for i in names_and_keys]
    keys = [i[1] for i in names_and_keys]
    return cols, [tuple([o.get(k, '') for k in keys])
                  for o in objs]


def get_client(obj):
    if hasattr(obj.app, 'client_manager'):
        return obj.app.client_manager.rca
    else:
        return obj.app.client


def wait_for_action_to_end(timeout, func, **kwargs):
    count = 0
    while count < timeout:
        if func(**kwargs):
            return True
        count += 1
        time.sleep(1)
    return False


def find_template_with_uuid(uuid, templates):
    return next(
        (
            template
            for template in templates
            if template['uuid'] == uuid
        ), None
    )



# Copyright 2019 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslotest import base

from vitrageclient import exceptions as exc
from vitrageclient.tests.utils import get_resources_dir
from vitrageclient.v1.template import Template

TEMPLATE_STRING = """
metadata:
 version: 3
 name: template1
 description: simple template
 type: standard
entities:
 alarm:
  name: cpu problem
 host:
  type: nova.host
scenarios:
 - condition: alarm [ on ] host
   actions:
     - set_state:
        state: ERROR
        target: host
"""


class TestTemplate(base.BaseTestCase):

    def test_validate_by_path(self):
        template_path = get_resources_dir() + '/template1.yaml'
        template = Template(mock.Mock())
        template.validate(path=template_path)

    def test_validate_by_nonexisting_path(self):
        template = Template(mock.Mock())
        self.assertRaises(IOError, template.validate,
                          path='non_existing_template_path.yaml')

    def test_validate_by_template(self):
        template = Template(mock.Mock())
        template.validate(template_str=TEMPLATE_STRING)

    def test_validate_by_nothing(self):
        template = Template(mock.Mock())
        self.assertRaises(exc.CommandError, template.validate)

    def test_add_by_path(self):
        template_path = get_resources_dir() + '/template1.yaml'
        template = Template(mock.Mock())
        template.add(path=template_path)

    def test_add_by_nonexisting_path(self):
        template = Template(mock.Mock())
        self.assertRaises(IOError, template.add,
                          path='non_existing_template_path.yaml')

    def test_add_by_template(self):
        template = Template(mock.Mock())
        template.add(template_str=TEMPLATE_STRING)

    def test_add_by_nothing(self):
        template = Template(mock.Mock())
        self.assertRaises(exc.CommandError, template.add)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from cliff import show
from vitrageclient.common import utils


# noinspection PyAbstractClass
class RcaShow(show.ShowOne):
    """Show the Root Cause Analysis for a certain alarm"""

    def get_parser(self, prog_name):
        parser = super(RcaShow, self).get_parser(prog_name)
        parser.add_argument('alarm_vitrage_id',
                            help='ID of an alarm')

        parser.add_argument('--all-tenants',
                            default=False,
                            dest='all_tenants',
                            action='store_true',
                            help='Shows alarms of all the tenants for the RCA')

        return parser

    @property
    def formatter_namespace(self):
        return 'vitrageclient.formatter.show'

    @property
    def formatter_default(self):
        return 'json'

    def take_action(self, parsed_args):
        alarm_id = parsed_args.alarm_vitrage_id
        all_tenants = parsed_args.all_tenants

        alarm = utils.get_client(self).rca.get(alarm_id=alarm_id,
                                               all_tenants=all_tenants)

        return self.dict2columns(alarm)



# Copyright 2017 Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from argparse import ArgumentParser
from argparse import ArgumentTypeError
from unittest import mock

# noinspection PyPackageRequirements
from datetime import datetime
from testtools import ExpectedException

from vitrageclient.tests.cli.base import CliTestCase
from vitrageclient.v1.cli.event import EventPost


# noinspection PyAttributeOutsideInit
class EventPostTest(CliTestCase):

    def setUp(self):
        super(EventPostTest, self).setUp()
        self.app = mock.Mock()
        self.event_post = EventPost(self.app, mock.Mock())

    def test_parsing_iso8601_with_not_a_date_string(self):
        self.assertRaises(ArgumentTypeError, self.event_post.iso8601, 'bla')

    def test_parsing_iso8601_in_a_good_format(self):
        self.event_post.iso8601('2014-12-13T12:44:21.123456')

    def test_iso8601_parsing_with_wrong_date_format(self):
        self.assertRaises(ArgumentTypeError, self.event_post.iso8601,
                          '2014/12/13 12:44:21')

    @mock.patch.object(ArgumentParser, "error")
    def test_parser_event_post_with_not_a_date_string(self, mock_parser):
        mock_parser.side_effect = self._my_parser_error_func
        parser = self.event_post.get_parser('vitrage event post')

        # noinspection PyCallByClass
        with ExpectedException(ArgumentTypeError,
                               'argument --time: -5 must be an iso8601 date'):
            parser.parse_args(args=['--type', 'bla',
                                    '--time', '-5',
                                    '--details', 'blabla'])

    @mock.patch.object(ArgumentParser, "error")
    def test_parser_event_post_type_required(self, mock_parser):
        mock_parser.side_effect = self._my_parser_error_func
        parser = self.event_post.get_parser('vitrage event post')

        # noinspection PyCallByClass
        with ExpectedException(ArgumentTypeError, r'.*--type'):
            parser.parse_args(args=['--details', 'blabla'])

    @mock.patch('vitrageclient.v1.cli.event.datetime')
    def test_parser_event_post_without_time_uses_time_now(self, dt_mock):
        current_time = datetime.now()
        dt_mock.now.return_value = current_time
        iso_time = current_time.isoformat()
        parser = self.event_post.get_parser('vitrage event post')
        args = parser.parse_args(args=['--type', 'bla',
                                       '--details', '{"blabla":[]}'
                                       ])

        with mock.patch.object(self.app.client.event, 'post') as poster_mock:
            self.event_post.take_action(args)

            poster_mock.assert_called_with(event_time=iso_time,
                                           details={'blabla': []},
                                           event_type='bla')



#  Copyright 2019 - Nokia Corporation
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
from cliff import lister

from vitrageclient.common import utils


class ServiceList(lister.Lister):
    """List all services"""

    def get_parser(self, prog_name):
        parser = super(ServiceList, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        service = utils.get_client(self).service.list()
        return utils.list2cols_with_rename(
            (
                ('Name', 'name'),
                ('Process Id', 'process'),
                ('Hostname', 'hostname'),
                ('Created At', 'created'),
            ), service)



#  Copyright 2018 - Nokia Corporation
#  #
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#  #
#       http://www.apache.org/licenses/LICENSE-2.0
#  #
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
import abc
from cliff.formatters import base
import six

from networkx.drawing.nx_pydot import write_dot
from networkx.readwrite.graphml import GraphMLWriter
from networkx.readwrite import json_graph

import networkx as nx


@six.add_metaclass(abc.ABCMeta)
class GraphFormatter(base.SingleFormatter):
    def add_argument_group(self, parser):
        pass

    def emit_one(self, column_names, data, stdout, _=None):
        data = {n: i for n, i in zip(column_names, data)}

        # vitrage properties are not standard
        # to convert with networkx we need to
        # use the standard properties
        # some converters have issues with multigraph
        # so disable it (currently we don't have real multigraphs)
        self._reformat(data)

        if nx.__version__ >= '2.0':
            graph = json_graph.node_link_graph(
                data, attrs={'name': 'graph_index'})
        else:
            graph = json_graph.node_link_graph(data)

        self._write_format(graph, stdout)

    @staticmethod
    def _reformat(data):
        for node in data['nodes']:
            name = node.pop('name', None)
            v_type = node['vitrage_type']
            if name and name != v_type:
                # if name and type the same
                # don't print twice its redundant
                # e.g openstack.cluster
                node[u'label'] = name + '\n' + v_type
            else:
                node[u'label'] = v_type

            # type list is not supported in some
            # formats
            GraphFormatter._list2str(node)

        data['multigraph'] = False

        for node in data['links']:
            node[u'label'] = node.pop('relationship_type')
            # used only in multigraph
            node.pop('key')

    @staticmethod
    def _list2str(node):
        for k, v in node.items():
            if type(v) == list:
                node[k] = str(v)

    @abc.abstractmethod
    def _write_format(self, graph, stdout):
        pass


class DOTFormatter(GraphFormatter):

    def _write_format(self, graph, stdout):
        write_dot(graph, stdout)


class GraphMLFormatter(GraphFormatter):

    def _write_format(self, graph, stdout):
        writer = GraphMLWriter(graph=graph)
        writer.dump(stdout)
